@@ -18,25 +18,24 @@ import numpy as np
 import base64
 import requests
 
+class ConsoleLogger:
+    def __init__(self, tag=''):
+        self.t = tag
 
-class NoLogger:
-    """
-    不输出日志
-    """
-    def log(self, tag, info, ts=0):
-        pass
-
-
-class ConsoleLogger(NoLogger):
+    def tag(self, t):
+        self.t = t
     """
     输出日志到控制台
     """
-    def log(self, tag, info, ts=0):
+    def info(self, info, ts=0):
         cur = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
         if ts:
-            print("{}\t{}\t{}\t耗时{}秒".format(cur, tag, info, round(ts, 3)))
+            print(f"{cur}\t{self.t}\t{info}\t耗时{round(ts, 3)}秒")
         else:
-            print("{}\t{}\t{}\t".format(cur, tag, info))
+            print(f"{cur}\t{self.t}\t{info}\t")
+
+    def error(self, info, ts=0):
+        self.log(self.t, info, ts)
 
     def __str__(self):
         return 'ConsoleLogger'
@@ -55,7 +54,7 @@ class MemoryRedup():
         return False
 
     def load(self, url):
-        print("新增去重 {}".format(url))
+        print(f"新增去重 {url}")
         self.pool.add(url)
 
     def __str__(self):
@@ -85,7 +84,7 @@ class RedisRedup(MemoryRedup):
         return False
 
     def load(self, url, name=NAME):
-        print("新增去重 {}".format(url))
+        print("新增去重 {url}".format(url=url))
         self.r.sadd(RedisRedup.NAME, url)
 
     def __str__(self):
@@ -137,6 +136,7 @@ class FilePipeline(ConsolePipeline):
     def __str__(self):
         return 'FilePipeline'
 
+
 class MySQLPipeline(ConsolePipeline):
     """
     结果输出到MySQL
@@ -156,24 +156,23 @@ class MySQLPipeline(ConsolePipeline):
         self.c = self.db.cursor()
         self.table_name = table
 
-    def save(self, values, table_name: str = ''):
+    def save(self, values, table_name: str = '')->None:
         """
         保存
         :param values:
         :return: 有错抛异常
         """
         sql = ''
-        t_name = table_name if table_name else  self.table_name
+        t_name = table_name if table_name else self.table_name
         try:
             fields = ",".join(values[0])
             for index in range(1, len(values)):
                 # 执行sql语句
-                sql = u"""INSERT INTO %s(%s) VALUES (%s)"""%(t_name, fields,  ",".join(['"'+pymysql.escape_string(v)+'"' for v in values[index]]))
+                sql = u"""INSERT INTO {name}({fields}) VALUES ({values})""".format(name=t_name, fields=fields,  values=",".join(['"'+pymysql.escape_string(v)+'"' for v in values[index]]))
                 self.c.execute(sql)
             # 提交到数据库执行
             self.db.commit()
         except Exception as e:
-            print(sql)
             # 如果发生错误则回滚
             self.db.rollback()
             raise e
@@ -213,7 +212,8 @@ class WordPressPipeline(ConsolePipeline):
                 }
                 auth = str.encode('{}:{}'.format(self.user, self.password))
                 headers = {'Authorization': 'Basic ' + str(base64.b64encode(auth), 'utf-8')}
-                resp = requests.post('http://{}/index.php/wp-json/wp/v2/posts'.format(self.host), headers=headers, data=data, timeout=5)
+                url = "http://{host}/index.php/wp-json/wp/v2/posts".format(host=self.host)
+                resp = requests.post(url, headers=headers, data=data, timeout=5)
                 if resp.status_code>201:
                     raise Exception(resp)
             except Exception as e:
@@ -252,7 +252,6 @@ class MemoryScheduler:
         :return:
         """
         ele = self.q.pop(0)
-        print("调度器  剩余容量{}  删除元素{}".format(self.len(), ele))
         return ele
 
     def len(self)->int:
@@ -262,8 +261,37 @@ class MemoryScheduler:
         return 'MemoryScheduler'
 
 
+class Hooker:
+    """
+    专门用于hook的类
+    """
+    def before_download(self, page)->None:
+        """
+        下载页面之前
+        :param page:
+        :return:
+        """
+        pass
+
+    def after_download(self, page)->None:
+        """
+        下载页面之后
+        :param page:
+        :return:
+        """
+        pass
+
+    def before_save(self, page)->None:
+        """
+        保存数据之前
+        :param page:
+        :return:
+        """
+        pass
+
+
 class Template:
-    def __init__(self, urls, expresses, next, fields_tag, fields, is_list, hooker):
+    def __init__(self, urls:list, expresses:dict, next:str, fields_tag:dict, fields:dict, is_list:bool, hooker: Hooker = Hooker()):
         """
         :param urls:
         :param expresses:
@@ -271,11 +299,10 @@ class Template:
         :param fields_tag: 表名，表示每个模板可以保存到不同的表中
         :param fields:
         :param is_list: True表示列表页，False表示实体页
-        :param hooker:
         """
-        assert urls
-        assert isinstance(urls, list)
-        assert expresses
+        assert urls, "urls参数不能空"
+        assert isinstance(urls, list), "urls参数不能空，必须是list"
+        assert expresses, "字段表达式不能空，必须是xpath表达式"
         self.urls = urls
         self.expresses = expresses
         self.next = next
@@ -286,7 +313,7 @@ class Template:
         self.child:Template = None  #下一级的模板
 
     def __str__(self):
-        return '({}Template:  urls={}  expresses={}  next={}  fields_tag={}  fields={} is_list={} hooker={} child={})'.format('列表' if self.is_list else '实体', self.urls, self.expresses, self.next, self.fields_tag, self.fields, self.is_list, self.hooker, self.child)
+        return f'(Template:  urls={self.urls}  expresses={self.expresses}  next={self.next}  fields_tag={self.fields_tag}  fields={self.fields} is_list={self.is_list} child={self.child})'
     __repr__ = __str__
 
 
@@ -301,13 +328,46 @@ class Page:
         self.parent = parent
         self.url = url
         self.template = template
+        self.whole_html = ''
         self.values: dict = {}    #抓取页面后的值，保存到这里
 
     def __str__(self):
-        return '(Page: parent={}  url={}  values={}  template={})'.format(self.parent, self.url, self.values, self.template)
+        return f'(Page: parent={self.parent}  url={self.url}  values={self.values}  template={self.template})'
     __repr__ = __str__
 
 
+class Parser:
+    def parse(self, page:Page):
+        pass
+
+
+class HtmlParser(Parser):
+    def parse(self, page)->None:
+        """
+        解析内容
+        :param page:
+        :return:
+        """
+        root_element = html.etree.HTML(page.whole_html)
+
+        ret = {}
+        for key, value in page.template.expresses.items():
+            value = str(value)
+            if value.startswith("/"):
+                content = root_element.xpath(value)
+                if not page.template.is_list:
+                    content = ["".join([item.strip() for item in content])]
+                ret[key] = content
+        # 抓取的记录条数
+        rows = len(list(ret.get(list(ret.keys())[0])))
+        # 表达式，不是/开头；那就是常量
+        for key, value in page.template.expresses.items():
+            value = str(value)
+            if not value.startswith("/"):
+                ret[key] = [str(value) for i in range(rows)]
+        page.values = ret
+
+
 class Downloader:
     """
     下载器
@@ -316,124 +376,41 @@ class Downloader:
     async def download(self, spider, page: Page) -> bool:
         return True
 
-    def __str__(self):
-        return 'Downloader'
-
-
-class SimpleDownloader(Downloader):
-    """
-    使用SimpleDownloader渲染后下载页面
-    """
-
-    async def download(self, spider, page: Page) -> bool:
-        try:
-            start = time.time()
-
-            response = None
-            try:
-                response = requests.get(page.url, timeout=5)
-            except:
-                spider.logger.log(spider.alias, '下载列表 {} 超时'.format(page.url), (time.time() - start))
-            content = response.content if response else ''
-            root_element = html.etree.HTML(content)
-            for key, value in page.template.expresses.items():
-                value = str(value)
-                if value.startswith("/"):
-                    content = root_element.xpath(value)
-                    if not page.template.is_list:
-                        content = ["".join(content)]
-                    page.values[key] = content
-            #抓取的记录条数
-            rows = len(list(page.values.get(list(page.values.keys())[0])))
-            #表达式，不是/开头；那就是常量
-            for key, value in page.template.expresses.items():
-                value = str(value)
-                if not value.startswith("/"):
-                    page.values[key] = [str(value) for i in range(rows)]
-
-            page.template.hooker.after_download(page)
-            spider.logger.log(spider.alias, '下载列表 {} 共计{}条 '.format(page.url, rows),(time.time() - start))
-        except:
-            spider.logger.log(spider.alias,
-                            '下载列表{}报错  {}'.format(page.url, 'traceback.format_exc():\n%s' % traceback.format_exc()))
+    async def click(self, spider, page: Page) -> bool:
         return True
 
     def __str__(self):
-        return 'SimpleDownloader'
+        return 'Downloader'
 
 
 class RenderDownloader(Downloader):
     """
     使用PyppeteerDownloader渲染后下载页面
     """
-
     async def download(self, spider, page: Page) -> bool:
         try:
             start = time.time()
-
-            browser_page = await spider.browser.newPage()
+            spider.browser_page = await spider.browser.newPage()
+            is_timeout = False
             try:
-                await browser_page.goto(page.url, options={'timeout':10000})
+                await spider.browser_page.goto(page.url, options={'timeout': 10000})
             except:
-                spider.logger.log(spider.alias, '下载列表 {} 超时'.format(page.url), (time.time() - start))
-            content = await  browser_page.content()
-            root_element = html.etree.HTML(content)
-            for key, value in page.template.expresses.items():
-                value = str(value)
-                if value.startswith("/"):
-                    content = root_element.xpath(value)
-                    if not page.template.is_list:
-                        content = ["".join([item.strip() for item in content])]
-                    page.values[key] = content
-            #抓取的记录条数
-            rows = len(list(page.values.get(list(page.values.keys())[0])))
-            #表达式，不是/开头；那就是常量
-            for key, value in page.template.expresses.items():
-                value = str(value)
-                if not value.startswith("/"):
-                    page.values[key] = [str(value) for i in range(rows)]
-            await browser_page.close()
-
+                is_timeout = True
+            content = await  spider.browser_page.content()
+            page.whole_html = content
+            spider.parser.parse(page)
+            await spider.browser_page.close()
             page.template.hooker.after_download(page)
-            spider.logger.log(spider.alias, '下载列表 {} 共计{}条 '.format(page.url, rows),(time.time() - start))
+            rows = len(list(page.values.get(list(page.values.keys())[0])))
+            spider.logger.info('下载 {url} {msg} 共计{count}条 '.format(url=page.url, msg='超时' if is_timeout else '', count=rows),(time.time() - start))
         except:
-            spider.logger.log(spider.alias,
-                            '下载列表{}报错  {}'.format(page.url, 'traceback.format_exc():\n%s' % traceback.format_exc()))
+            spider.logger.error('下载 {url}报错  {msg}'.format(url=page.url, msg='traceback.format_exc():\n%s' % traceback.format_exc()))
         return True
 
     def __str__(self):
         return 'PyppeteerDownloader'
 
 
-class Hooker:
-    """
-    专门用于hook的类
-    """
-    def before_download(self, page: Page)->None:
-        """
-        下载页面之前
-        :param page:
-        :return:
-        """
-        pass
-
-    def after_download(self, page: Page)->None:
-        """
-        下载页面之后
-        :param page:
-        :return:
-        """
-        pass
-
-    def before_save(self, page: Page)->None:
-        """
-        保存数据之前
-        :param page:
-        :return:
-        """
-        pass
-
-
 class Spider:
     url_regex = re.compile(
         r'^(?:http|ftp)s?://'  # http:// or https://
@@ -443,29 +420,60 @@ class Spider:
         r'(?::\d+)?'  # optional port
         r'(?:/?|[/?]\S+)$', re.IGNORECASE)
 
-    def __init__(self, alias: str, downloader: Downloader = RenderDownloader(), redup: MemoryRedup = MemoryRedup(), scheduler: MemoryScheduler = MemoryScheduler(), pipeline: ConsolePipeline=ConsolePipeline(), logger: ConsoleLogger=ConsoleLogger()):
+    def __init__(self, alias: str, downloader: Downloader = RenderDownloader(), redup: MemoryRedup = MemoryRedup(), scheduler: MemoryScheduler = MemoryScheduler(), parser: Parser = HtmlParser(), pipeline: ConsolePipeline = ConsolePipeline(), logger: ConsoleLogger = ConsoleLogger()):
         """
         实例化爬虫类，各个参数很重要，需要认真填写
         :param alias: 网站名称，方便记忆
         :param pattern: 运行模式，可选值有1、2。值1表示使用简洁模式，值2表示使用渲染模式
         :param redup: 判断重复的类，必须创建对象，可选类有MemoryRedup、RedisRedup
         :param scheduler: 调度器类，必须创建对象，可选类有MemoryScheduler
+        :param parser: 条目解析器
         :param pipeline: 持久化的类，必须创建对象，可选类有ConsoleDao、FilePipeline、MySQLPipeline、WordPressPipeline
         :param logger: 日志类，必须创建对象，可选类有NoLogger、ConsoleLogger
         """
         self.pid = os.getpid()
-        logger.log(tag='爬虫实例 进程={}'.format(self.pid), info='downloader={} redup={} scheduler={} pipeline={} logger={}'.format(downloader, redup, scheduler, pipeline, logger))
+        self.logger = logger
+        self.logger.tag(alias)
         self.alias = alias
         self.downloader = downloader
         self.redup = redup
         self.scheduler = scheduler
+        self.parser = parser
         self.pipeline = pipeline
-        self.logger = logger
         self.browser = asyncio.get_event_loop().run_until_complete(launch({'headless': True, 'args': ['--no-sandbox', '--disable-setuid-sandbox'], 'dumpio': True, 'slowMo': 1}))
+        self.browser_page = None
+        self.event_loop = asyncio.get_event_loop()
         self.template = None    #保存本页面对应的模板
         self.save_count = 0 #保存成功的数量
 
-    def page(self, urls:Union[list, str] = '', is_list: bool = False, expresses: dict = {}, fields_tag: str = '', fields: dict = {}, next: str = '', hooker: Hooker = Hooker()):
+    def list(self, urls: Union[list, str] = '', expresses: dict = {}, fields_tag: str = '', fields: dict = {}, next: str = '', hooker: Hooker = Hooker()):
+        """
+        列表
+        :param urls:
+        :param expresses:
+        :param fields_tag:
+        :param fields:
+        :param next:
+        :param hooker:
+        :return:
+        """
+        # self.logger.info('list(...)参数 urls={urls} expresses={expresses} fields_tag={fields_tag} fields={fields} next={next} hooker={hooker}'.format(urls=urls, expresses=expresses, fields_tag=fields_tag, fields=fields, next=next, hooker=hooker))
+        return self.__page(urls=urls, is_list=True, expresses=expresses, fields_tag=fields_tag, fields=fields, next=next, hooker=hooker)
+
+    def page(self, urls: Union[list, str] = '', expresses: dict = {}, fields_tag: str = '', fields: dict = {}, hooker: Hooker = Hooker()):
+        """
+        单页
+        :param urls:
+        :param expresses:
+        :param fields_tag:
+        :param fields:
+        :param hooker:
+        :return:
+        """
+        # self.logger.info('page(...)参数 urls={urls} expresses={expresses} fields_tag={fields_tag} fields={fields} next={next} hooker={hooker}'.format(urls=urls, expresses=expresses, fields_tag=fields_tag, fields=fields, next=next, hooker=hooker))
+        return self.__page(urls=urls, is_list=False, expresses=expresses, fields_tag=fields_tag, fields=fields, hooker=hooker)
+
+    def __page(self, urls: Union[list, str] = '', is_list: bool = False, expresses: dict = {}, fields_tag: str = '', fields: dict = {}, next: str = '', hooker: Hooker = Hooker()):
         """
         抓取信息配置
         :param urls: 被抓取的url列表，可以是list，也可以是str
@@ -480,9 +488,8 @@ class Spider:
         :param hooker: Hooker 用于hook
         :return: self
         """
-        self.logger.log(self.alias, 'page(...)参数 urls={} expresses={} fields_tag={} fields={} next={} is_list={} hooker={}'.format(urls, expresses, fields_tag, fields, next, is_list, hooker))
         urls = [urls] if isinstance(urls, str) else urls
-        t = Template(urls, expresses, next, fields_tag, fields, is_list, hooker)
+        t = Template(urls=urls, expresses=expresses, next=next, fields_tag=fields_tag, fields=fields, is_list=is_list, hooker=hooker)
         if self.template:
             self.template.child = t
         else:
@@ -495,16 +502,15 @@ class Spider:
         :param page:
         :return: 重复url，返回False；否则，返回True
         """
-        self.logger.log(self.alias, '__download(...)参数 {}'.format(page))
+        self.logger.info('__download(...)')
         page.template.hooker.before_download(page)
-        url = page.url
-        if self.redup.loaded(url):
+        assert re.match(Spider.url_regex, page.url), "不是合法的url格式"
+        if self.redup.loaded(page.url):
             return False
-        assert re.match(Spider.url_regex, url)
         return await self.downloader.download(self, page)
 
     def __pre_save(self, page: Page)->None:
-        self.logger.log(self.alias, '__pre_save(...)参数 {}'.format(page))
+        # self.logger.info('__pre_save(...)参数')
 
         #使用fields内容
         items = {}
@@ -527,7 +533,7 @@ class Spider:
                 m.append(value)
             matrix.append(m)
         if page.template.is_list and len(v_len) != 1:
-            raise Exception(self.alias +' 抓取字段的数量不一致 ' + str(page.values))
+            raise Exception(self.alias +' 抓取字段的数量不一致 ' + str(v_len))
 
         # 后面会根据key取值，所以Page对象增加一个新的属性
         page.matrix = np.array(matrix).T.tolist()
@@ -561,22 +567,24 @@ class Spider:
         :param page:
         :return:
         """
-        self.logger.log(self.alias, '__save(...)参数 {}'.format(page))
+        self.__pre_save(page)
+        # self.logger.info('__save(...)参数 {}'.format(page))
         try:
             self.pipeline.save(page.matrix, page.template.fields_tag)
             self.save_count = self.save_count+1
             return True
         except:
-            self.logger.log(self.alias, '保存报错 {}'.format('traceback.format_exc():\n%s' % traceback.format_exc()))
+            self.logger.error('保存报错 {msg}'.format(msg='traceback.format_exc():\n%s' % traceback.format_exc()))
             return False
 
     def __after_save(self, page: Page, flag:bool)->None:
         """
         保存后，修改调度器信息
-        :param node:
+        :param page:
+        :param flag:
         :return:
         """
-        self.logger.log(self.alias, '__after_save(...)参数 page={} flag={}'.format(page, flag))
+        # self.logger.info('__after_save(...)参数 page={} flag={}'.format(page, flag))
         # 1、成功操作后，加入到去重队列
         if flag:
             self.redup.load(page.url)
@@ -589,46 +597,49 @@ class Spider:
         # 4、删除头元素
         self.scheduler.remove_head()
         # 5、日志
-        self.logger.log(self.alias, '成功保存数据{}条 '.format(self.save_count))
+        self.logger.info('成功保存数据{count}条 '.format(count=self.save_count))
 
     def run(self):
         """
         开始运行
         :return:
         """
-        self.logger.log(self.alias, 'run(...)开始运行')
+
+        self.logger.info('run(...)开始运行')
+        try:
+            self.__run()
+            self.logger.info('运行结束')
+        except:
+            self.logger.error('运行报错 {msg}'.format(msg='traceback.format_exc():\n%s' % traceback.format_exc()))
+        finally:
+            self.logger.info('清理环境')
+            self.__kill()
+
+    def __run(self):
         # 生成种子
         for url in self.template.urls:
             self.scheduler.put(Page(parent=None, url=url, template=self.template))
 
-        event_loop = asyncio.get_event_loop()
-        try:
-            while self.scheduler.len():
-                page = self.scheduler.head()
-                f = asyncio.ensure_future(self.__download(page))
-                event_loop.run_until_complete(f)
-                normal_flag = True
-                if f.result() and page.template.fields:
-                    self.__pre_save(page)
-                    normal_flag = self.__save(page)
-                self.__after_save(page, normal_flag)
-            self.logger.log(self.alias, '运行结束')
-        except:
-            self.logger.log(self.alias, '运行报错 {}'.format('traceback.format_exc():\n%s' % traceback.format_exc()))
-        finally:
-            self.logger.log(self.alias, '清理环境')
-            self.__kill()
+        while self.scheduler.len():
+            page = self.scheduler.head()
+            f = asyncio.ensure_future(self.__download(page))
+            self.event_loop.run_until_complete(f)
+
+            normal_flag = True
+            if f.result() and page.template.fields:
+                normal_flag = self.__save(page)
+            self.__after_save(page, normal_flag)
 
     def __kill(self):
         """
         清理环境
         :return:
         """
-        self.logger.log(self.alias, '关闭浏览器')
+        self.logger.info('关闭浏览器')
         try:
             # win平台
             if platform.system() == 'Windows':
-                subprocess.Popen("taskkill /pid {} /f".format(self.pid), shell=True)
+                subprocess.Popen("taskkill /pid {pid} /f".format(pid=self.pid), shell=True)
             # linux平台
             if platform.system() == 'Linux':
                 os.system("ps -ef |grep chrome |awk '{print $2}'|xargs kill -9")
